@@ -139,6 +139,21 @@ test_that("cleaning works with unnamed rows and/or columns", {
 })
 
 
+test_that("cleaning works with tolerance", {
+  unnamed <- matrix(c(1, 2, 0.1,
+                      3, 4, -0.1,
+                      5, 6, 0.05,
+                      0.01, -0.01, -0.05), nrow = 4, byrow = TRUE)
+  expect_equal(clean_byname(unnamed, tol = 0.1), matrix(c(1, 2,
+                                                          3, 4, 
+                                                          5, 6), nrow = 3, byrow = TRUE))
+  # Tighten tolerance to get different result.
+  expect_equal(clean_byname(unnamed, tol = 0.0999), matrix(c(1, 2, 0.1, 
+                                                             3, 4, -0.1,
+                                                             5, 6, 0.05), nrow = 3, byrow = TRUE))
+})
+
+
 ###########################################################
 context("Is zero")
 ###########################################################
@@ -332,16 +347,16 @@ test_that("renaming rows to prefix or suffix works as expected", {
               dimnames = list(c("a -> b", "r2", "r3"), c("c1", "c2")))
   expected <- m
   rownames(expected) <- c("a", "r2", "r3")
-  actual <- rename_to_pref_suff_byname(m, sep = " -> ", keep = "prefix", margin = 1)
+  actual <- rename_to_pref_suff_byname(m, keep = "prefix", margin = 1, notation = arrow_notation())
   expect_equal(actual, expected)
   
   expected <- m
   rownames(expected) <- c("b", "r2", "r3")
-  actual <- rename_to_pref_suff_byname(m, sep = " -> ", keep = "suffix", margin = 1)
+  actual <- rename_to_pref_suff_byname(m, keep = "suffix", margin = 1, notation = arrow_notation())
   expect_equal(actual, expected)
   
   # Check that renaming works for a list
-  actual <- rename_to_pref_suff_byname(list(m, m), sep = " -> ", keep = "suffix", margin = 1)
+  actual <- rename_to_pref_suff_byname(list(m, m), keep = "suffix", margin = 1, notation = arrow_notation())
   expect_equal(actual, list(expected, expected))
 })
 
@@ -353,21 +368,21 @@ test_that("renaming columns to prefix or suffix works as expected", {
               dimnames = list(c("a -> b", "r2", "r3"), c("a -> b", "c -> d")))
   expected <- m
   colnames(expected) <- c("a", "c")
-  actual <- rename_to_pref_suff_byname(m, sep = " -> ", keep = "prefix", margin = 2)
+  actual <- rename_to_pref_suff_byname(m, keep = "prefix", margin = 2, notation = arrow_notation())
   expect_equal(actual, expected)
   
   expected <- m
   colnames(expected) <- c("b", "d")
-  actual <- rename_to_pref_suff_byname(m, sep = " -> ", keep = "suffix", margin = 2)
+  actual <- rename_to_pref_suff_byname(m, keep = "suffix", margin = 2, notation = arrow_notation())
   expect_equal(actual, expected)
   
   # Check that renaming works for a list
-  actual <- rename_to_pref_suff_byname(list(m, m), sep = " -> ", keep = "suffix", margin = 2)
+  actual <- rename_to_pref_suff_byname(list(m, m), keep = "suffix", margin = 2, notation = arrow_notation())
   expect_equal(actual, list(expected, expected))
   
   # Check that row and column types are preserved
   m <- m %>% setrowtype("Rows") %>% setcoltype("Cols")
-  res <- rename_to_pref_suff_byname(m, sep = " -> ", keep = "suffix")
+  res <- rename_to_pref_suff_byname(m, keep = "suffix", notation = arrow_notation())
   expect_equal(rowtype(res), "Rows")
   expect_equal(coltype(res), "Cols")
 })
@@ -382,27 +397,47 @@ test_that("renaming rows and columns to prefix or suffix works as expected", {
   rownames(expected) <- c("a", "r2", "r3")
   colnames(expected) <- c("a", "c")
   # Default is margin = c(1, 2)
-  actual <- rename_to_pref_suff_byname(m, sep = " -> ", keep = "prefix")
+  actual <- rename_to_pref_suff_byname(m, keep = "prefix", notation = arrow_notation())
   expect_equal(actual, expected)
   
   expected <- m
   rownames(expected) <- c("b", "r2", "r3")
   colnames(expected) <- c("b", "d")
-  actual <- rename_to_pref_suff_byname(m, sep = " -> ", keep = "suffix")
+  actual <- rename_to_pref_suff_byname(m, keep = "suffix", notation = arrow_notation())
   expect_equal(actual, expected)
   
   # Check that renaming works for a list
-  actual <- rename_to_pref_suff_byname(list(m, m), margin = list(c(1, 2)), sep = " -> ", keep = "suffix")
+  actual <- rename_to_pref_suff_byname(list(m, m), margin = list(c(1, 2)), keep = "suffix", notation = arrow_notation())
   expect_equal(actual, list(expected, expected))
 
   # Check that row and column types are preserved
   m <- m %>% setrowtype("Rows") %>% setcoltype("Cols")
-  res <- rename_to_pref_suff_byname(m, sep = " -> ", keep = "prefix")
+  res <- rename_to_pref_suff_byname(m, keep = "prefix", notation = arrow_notation())
   expect_equal(rowtype(res), "Rows")
   expect_equal(coltype(res), "Cols")
 })
 
 
+test_that("renaming rows and cols to pref and suff also changes rowtype and coltype", {
+  m <- matrix(c(1, 2, 
+                3, 4, 
+                5, 6), nrow = 3, byrow = TRUE, 
+              dimnames = list(c("a -> b", "c -> d", "e -> f"), c("g -> h", "i -> j"))) %>% 
+    setrowtype("Industry -> Product") %>% setcoltype("Product -> Industry")
+  res <- rename_to_pref_suff_byname(m, keep = "prefix", notation = arrow_notation())
+  expect_equal(rownames(res), c("a", "c", "e"))
+  expect_equal(colnames(res), c("g", "i"))
+  expect_equal(rowtype(res), "Industry")
+  expect_equal(coltype(res), "Product")
+
+  res2 <- rename_to_pref_suff_byname(m, keep = "suffix", notation = arrow_notation())
+  expect_equal(rownames(res2), c("b", "d", "f"))
+  expect_equal(colnames(res2), c("h", "j"))
+  expect_equal(rowtype(res2), "Product")
+  expect_equal(coltype(res2), "Industry")
+})
+
+
 test_that("setting identical row/col names is OK", {
   m <- matrix(c(1, 2, 
                 3, 4, 
@@ -412,7 +447,7 @@ test_that("setting identical row/col names is OK", {
   rownames(expected) <- c("a", "a", "r3")
   colnames(expected) <- c("a", "a")
   # The next call will create duplicate row names and column names in m. 
-  actual <- rename_to_pref_suff_byname(m, sep = " -> ", keep = "prefix")
+  actual <- rename_to_pref_suff_byname(m, keep = "prefix", notation = arrow_notation())
   # Interestingly the command View(actual) or View(expected)
   # shows row names that aren't true. 
   # The 2nd row of actual and expected is shown as "a.1", 
@@ -431,51 +466,49 @@ test_that("renaming with full prefix identifiers works as expected.", {
   expected <- m
   dimnames(expected) <- list(c("a", "c", "e"), c("g", "i"))
   # The next call will create duplicate row names and column names in m. 
-  actual <- rename_to_pref_suff_byname(m, keep = "prefix", prefix_open = "", prefix_close = " [")
+  actual <- rename_to_pref_suff_byname(m, keep = "prefix", notation = bracket_notation())
   expect_equal(actual, expected)
   
   expected2 <- m
   dimnames(expected2) <- list(c("a", "c", "e"), c("g [h]", "i [j]"))
-  actual2 <- rename_to_pref_suff_byname(m, keep = "prefix", margin = 1, prefix_open = "", prefix_close = " [")
+  actual2 <- rename_to_pref_suff_byname(m, keep = "prefix", margin = 1, notation = bracket_notation())
   expect_equal(actual2, expected2)
   
   expected3 <- m
   dimnames(expected3) <- list(c("a [b]", "c [d]", "e [f]"), c("g", "i"))
-  actual3 <- rename_to_pref_suff_byname(m, keep = "prefix", margin = 2, prefix_open = "", prefix_close = " [")
+  actual3 <- rename_to_pref_suff_byname(m, keep = "prefix", margin = 2, notation = bracket_notation())
   expect_equal(actual3, expected3)
   
   expected4 <- m
   dimnames(expected4) <- list(c("b", "d", "f"), c("h", "j"))
-  actual4 <- rename_to_pref_suff_byname(m, keep = "suffix", suffix_open = " [", suffix_close = "]")
+  actual4 <- rename_to_pref_suff_byname(m, keep = "suffix", notation = bracket_notation())
   expect_equal(actual4, expected4)
 
   expected5 <- m
   dimnames(expected5) <- list(c("b", "d", "f"), c("g [h]", "i [j]"))
-  actual5 <- rename_to_pref_suff_byname(m, keep = "suffix", margin = 1, suffix_open = " [", suffix_close = "]")
+  actual5 <- rename_to_pref_suff_byname(m, keep = "suffix", margin = 1, notation = bracket_notation())
   expect_equal(expected5, actual5)
 
   expected6 <- m
   dimnames(expected6) <- list(c("a [b]", "c [d]", "e [f]"), c("h", "j"))
-  actual6 <- rename_to_pref_suff_byname(m, keep = "suffix", margin = 2, suffix_open = " [", suffix_close = "]")
+  actual6 <- rename_to_pref_suff_byname(m, keep = "suffix", margin = 2, notation = bracket_notation())
   expect_equal(expected6, actual6)
   
   # Try with a list
-  actual_list <- rename_to_pref_suff_byname(list(m, m), keep = "prefix", margin = 1, prefix_open = "", prefix_close = " [")
+  actual_list <- rename_to_pref_suff_byname(list(m, m), keep = "prefix", margin = 1, notation = bracket_notation())
   expect_equal(actual_list[[1]], expected2)
   expect_equal(actual_list[[2]], expected2)
   
   # Try in a data frame
   DF <- tibble::tibble(m = list(m, m, m, m, m, m), keep = c("prefix", "prefix", "prefix", "suffix", "suffix", "suffix"), 
                        margin = list(c(1, 2), 1, 2, c(1, 2), 1, 2),
-                       prefix_open = "", prefix_close = " [", 
-                       suffix_open = " [", suffix_close = "]", 
+                       notation = list(bracket_notation()),
                        expected = list(expected, expected2, expected3, expected4, expected5, expected6))
   
   res <- DF %>% 
     dplyr::mutate(
       actual = rename_to_pref_suff_byname(m, keep = keep, margin = margin, 
-                                          prefix_open = prefix_open, prefix_close = prefix_close,
-                                          suffix_open = suffix_open, suffix_close = suffix_close)
+                                          notation = notation)
     )
   expect_equal(res$actual, res$expected)
 })
